'''
Created on May 20, 2013

@author: mwallers
'''
"""

Attempts to find mobile versions of pages by looking at previously
found pages and inferring the transformations from desktop urls to
mobile urls. Generates several url guesses and tests them out. If one
of them is found to be valid, it is tested to see if it is significantly
different from the desktop page before being recorded as a mobile site.

This implementation relies heavily on Google's diff-match-patch library:
http://code.google.com/p/google-diff-match-patch/

This also makes use of Ben Hoyt's soft 404 detection script:
http://blog.brush.co.nz/2008/01/soft404s/


input: fixed (desktop) urls; fileLocation of where the content folder is located
output: fixed urls of which a mobile version was not found

along the way it records to file if the urls were found or not to the respective files

"""


"""my version of a description:
just look for any variation of the url and keep trying to access the webpage
and try and detect soft 404 errors along the way. if it does return as a real page then
make sure it is a page that is similar in url to the one searched for. 
	--- either that or let the mobileIdentifier take search pages into account 
		in determining whether a page is really a mobile page or not. 
"""
import tldextract
import config
import urllib2
#from ..util.request import requestPage
from ..util.confirmURLDifference import differenceExists, get_last_score
from ..soft404 import is_dead
import socket
from ..util.logger import log
from ..util import record 

def usePatterns(url):   #should be in format with http://...
	possibleURLs = []
	
	#using tldextract from https://github.com/john-kurkowski/tldextract
	ext = tldextract.extract(url)
	ending = ""
	mainBody = ""
	subpart = ""

	if ext.subdomain == 'www' or ext.subdomain == "":
		ending = ext.tld
		mainBody = ext.domain
		subpart = "www."
	elif ext.subdomain.find("www.") == 0 :
		#special case like for www.espn.go.com. Where ext.subdomain= 'www.espn', ext.domain = 'go', ext.tld = 'com'
		newSubdomain = ext.subdomain.replace("www.", "")
		mainBody = newSubdomain 
		ending = ext.domain + "." + ext.tld
		subpart = "www."
	else:
		ending = ext.domain + "." + ext.tld
		mainBody = ext.subdomain
		if "www" in mainBody:
			mainBody.replace("www", "")
	
	#keep track of http:// or https:// to get attempted mobile url with the same level of security
	position = url.find('://')
	beginning = ""
	if position == -1:
		beginning = 'http://'  #add this as the beginning
	else:    
		beginning = url[0:position+len('://')]
			
		possible = beginning + 'm.' + mainBody + "." + ending
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '/index.m.html'
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + '.mobi'
		possibleURLs.append(possible)
		
		possible = beginning + 'mobi.' + mainBody + "." + ending
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '/mobile'
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '/m/'
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '?m=1'
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '/forum.php?mobile=yes'
		possibleURLs.append(possible)
		
		possible = beginning + subpart + mainBody + "." + ending + '?mobile=yes'
		possibleURLs.append(possible)
	
	return possibleURLs


#this function looks for links that should resemble: <link rel="alternate" media="only screen and (max-width: 640px)" href="http://m.example.com/page-1" >  
#    (though the media part is unimportant here)
def lookForAlternateLinksForGoogleVersion(url, content): #a fixed url, not mobile    
	
	mobileLinks = []
	for line in content: #assuming the content is a series of lines
		correctLink = "<link rel='alternate'"
		if correctLink in line:
			substr = line[0:] #copy all of line into substr
			position = line.find(correctLink) + 1
			while not position == -1:                    
				link = ""                
				substr = substr[position:]
				position = substr.find('href="') + 1
				if position == -1:
					position = substr.find("href='") + 1
					substr = substr[position:]
					position = substr.find("'") 
					link = substr[:position]
				else:
					substr = substr[position:]
					position = substr.find('"') 
					link = substr[:position]
								  
				possibleMobileURL = link 
				print "**possible url: "+ link                
				mobileLinks.append(possibleMobileURL)                
				position = substr.find(correctLink) + 1 #for next round in case many links on one line. though there shouldn't be
	return mobileLinks                

#this function looks for links that should resemble: <link rel="canonical" href="http://www.example.com/page-1" >
def lookForCanonicalLinksForGoogleVersion(mobileURL, mobileContent):
	mobileLinks = []
	for line in mobileContent: #assuming the content is a series of lines
		correctLink = "<link rel='canonical'"
		if correctLink in line:
			substr = line[0:] #copy all of line into substr
			position = line.find(correctLink) + 1
			while not position == -1:                    
				link = ""                
				substr = substr[position:]
				position = substr.find('href="') + 1
					#this is in case tick marks are used around the link
				if position == -1:
					position = substr.find("href='") + 1
					substr = substr[position:]
					position = substr.find("'") 
					link = substr[:position]
					#this is for when actual quotes are used around the link
				else:
					substr = substr[position:]
					position = substr.find('"') 
					link = substr[:position]
								  
				possibleMobileURL = link 
				print "**possible url: "+ link                
				mobileLinks.append(possibleMobileURL)                
				position = substr.find(correctLink) + 1 #for next round in case many links on one line. though there shouldn't be
	return mobileLinks      
	
	
def getMobileVersion(urls, fileLocation):
	remainingURLs = []
	for url in urls:  
		'''commented out section is below:'''
		
		followsRulesFromGoogle = True
		log('differingURLs find %s' % url)
		possibleURLs = usePatterns(url)
		foundMobile = False
		found = False
		foundForGoogle = False
		
		content = None
		filename = None
		try:
				filename = config.getDomain(url, False, True) #**************************************************

				filename = fileLocation + filename
				htmlfile = open(filename + '-fixed.html', 'r')
				content = htmlfile.read()
				htmlfile.close()
		except IOError as e:
				log("error opening the fixed files...")
				print "I/O error({0}): {1}".format(e.errno, e.strerror)
		except:
				print "unexpected error"
				log ("\tunexpected error!")
		'''
		if config.isGoogleVersion:
			mobileLinks = []
			mobileLinks = lookForAlternateLinksForGoogleVersion(url, content)
			if not mobileLinks: #if empty list
				followsRulesFromGoogle = False
				record.printGoogleDifferent = True
				record.record_notfound(url, config.notFoundFilenameGoogleVersion, config.statsFilenameGoogleVersion, "lack of proper links for alternate and canonical pages")
				record.printGoogleDifferent = False
				log("\t-> not found because lack of proper links for alternate and canonical pages")
				log("\t->-> missing use of alternate links")
		'''   
		for mobileURL in possibleURLs:
			found = False  #<for google version. it might have to go through several times even when then 'normal' version has found a mobile version of the url and has returned. 
			try:
				log ("\tChecking possible mobile url: " + mobileURL)
				#check for soft 404 errors:
		   
				#Poke the URL (this will throw an exception if the URL
				#is invalid, which happens most of the time)            
				urllib2.urlopen(mobileURL, timeout=20) #timeout is to keep it from taking forever
				
				#Check for soft 404s
				log('\tchecking for soft 404... ', False)
	
				if is_dead(mobileURL):
					log('soft 404 detected')
					continue
				else:
					log('not detected')
				
				#If we've got through the above stuff, we can assume
				#the url at least works, so let's see if it's different from the fixed page
				#    get mobile content using request
				urlFilename = config.getDomain(url, False, True)
				mobileContent = None

				needRepeat = True
				isFirstAttempt = True
				#log('\trequesting mobile...')
				'''
				while needRepeat:
					isError = False
					if not isFirstAttempt:
						log('\tre-attempting: requesting mobile... ', False)

					mobileStatus, mobileFinalURL = requestPage('mobile', mobileURL, urlFilename) #< the last argument is to save under url and NOT mobileUrl
					if mobileStatus == 'success':
					'''
				htmlFilename = ""
				try:
					#use fixed url for where the file is saved, not the mobile url
					mobileFilename = config.getDomain(url, False, True) 

					mobileFilename = fileLocation + mobileFilename
					htmlFilename = mobileFilename +  '-mobile.html'
					#htmlfile = open(mobileFilename + '-mobile.html', 'r')

					#the filename refers to where it was specifially saved (at the ficed url's location
					htmlfile = open(htmlFilename, 'r')
					mobileContent = htmlfile.read()
					htmlfile.close()
				except IOError as e:
					log("error opening the mobile files...")
					print ("failed file name is: " , htmlFilename)
					print "I/O error({0}): {1}".format(e.errno, e.strerror)
				except:
					print "unexpected error"
					log ("\tunexpected error!")
				'''
						 else:
							 needRepeat = False #no errors occurred
					elif mobileStatus == 'fail': #dont want to attempt again if the last attempt ended in a hang
						if isFirstAttempt:
							isFirstAttempt = False
						else:
							needRepeat = False
					else:  
						log ("error retrieving mobile page. status not success...")
						needRepeat = False
						isError = True
						continue
				if isError == True:
					#record.record_notfound(url, config.neverFoundFilename, None, None, True)
					continue
				'''
				if differenceExists(url,content, None, None, mobileURL, mobileContent, None, None, fileLocation): #takes fixedURL, fixedContent, fixedStylesheets, fixedJs, mobileURL, mobileContent, mobileStylesheets, mobileJs, fileLocation
					found = True
				   
				#check for alternate and canonical links. sort of implemented below
				if config.isGoogleVersion and found: #only look if google version and the mobile url is the correct mobile url
					mobileLinks = []
					mobileLinks = lookForAlternateLinksForGoogleVersion(url, content)
					if not mobileLinks: #if empty list
						followsRulesFromGoogle = False
						record.printGoogleDifferent = True
						record.record_notfound(url, config.notFoundFilenameGoogleVersion, config.statsFilenameGoogleVersion, "lack of proper links for alternate and canonical pages")
						record.printGoogleDifferent = False
						log("\t-> not found because lack of proper links for alternate and canonical pages")
						log("\t->-> missing use of alternate links")
						
				if config.isGoogleVersion and found:# and followsRulesFromGoogle:
					mobileLinksCanonical = lookForCanonicalLinksForGoogleVersion(mobileURL, mobileContent)
					if not mobileLinksCanonical: #empty list
						log("\t->-> missing use of canonical links")
					if mobileURL in mobileLinks:
						if url in mobileLinksCanonical:
							if followsRulesFromGoogle:
								foundForGoogle = True
						else:
							log("\t->-> improper use of canonical links")
					elif mobileLinks and not mobileURL in mobileLinks:
						log("\t->-> improper use of alternate links")
						if not url in mobileLinksCanonical:
							log("\t->-> improper use of canonical links")
						
						
									
			#Thrown if the guessed url is invalid (quite likely)
			except urllib2.URLError, e:
				pass
			except urllib2.HTTPError:
				pass
			except socket.error:
				pass
			except socket.timeout:
				log('timeout on invalid mobile url from differingURLs.py')
			except Exception as e:
				#Some exception we haven't checked for above
				#Probably should find out what it is
				log('%s %s' % (mobileURL, str(type(e))))
			
			   
			'''
			TODO: need to VERIFY that the page does not say page not found on it!! and that it is not a search page!!
			
			verify page returned has at least similar domain to what was initially searched for!!!
			'''

			#output to record if found or not found to the respective files (Google version or not)
			if found: 
				record.record_found(url, mobileURL, config.foundFilename, config.statsFilename)
				foundMobile = True
				#cleanup directories to make room:
				config.cleanupUnusedDirectories (url)
				
				
				#look through the content and see if it is attempting to use client-side redirect
				lines = mobileContent.split('\n')
				for line in lines:
					if line.find("navigator.userAgent") >= 0:
						#flag it as trying to use client-side reidrect
						log("\t\tFLAG: attempting to redirect on client-side")
						break
				break  #and exit the loop of possible URLs
					

		#end for loop
		if config.isGoogleVersion and followsRulesFromGoogle:
			if foundForGoogle:  #found for Google version
				record.record_found(url, mobileURL, config.foundFilenameGoogleVersion)
				foundMobile = True
				#cleanup directories to make room:
				config.cleanupUnusedDirectories (url)
			else:   #not found for Google version
				record.record_notfound(url, config.notFoundFilenameGoogleVersion, config.statsFilenameGoogleVersion, "lack of proper links for alternate and canonical pages")
				log("\t-> not found because lack of proper links for alternate and canonical pages")        
		if not found:
			record.record_notfound(url, config.notFoundFilename)
			  
		if not foundMobile:
			remainingURLs.append(url)
		
		print "made it to here in differingURLS with url: " + url + "\n"
	return remainingURLs
	#return urls