'''
Created on May 20, 2013

@author: mwallers
'''
"""
Looks for the presence of CSS media queries to indicate a mobile page.

Only media queries that are based on width count for mobile sites. I
believe this is the most common way of querying for mobile devices.
There are admitted flaws in this approach, but it seems mostly workable.


input: fixed (desktop) urls; fileLocation of where the content folder is located
output: fixed urls of which a mobile version was not found

along the way it records to file if the urls were found or not to the respective files

"""

"""
TODO: check into Accept headers  ((http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html)) for media specifications

"""
from ..util.logger import log
import config
import re
import urllib2
#from ..util.request import requestPage
from ..util import record #record_found, record_notfound, record_error_MediaQueries, record_error, printGoogleDifferent

#Same UA as used by request.py, used when this module directly requests a page
MOBILE_UA="Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_1 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8B117 Safari/6531.22.7 (compatible; mediaqueries/1.0; +http://cs.harding.edu)"

def findMediaQueriesInStylesheets(sheets, url):
	foundMedia, foundMediaWidth = False, False
	
	for stylesheet in sheets:
		if stylesheet:  #if it is not blank or none
			#log('\t\trequesting %s' % stylesheet)
			if foundMediaWidth:
				break  #stop looking through the stylesheets since we have what we came for  :)

			try:
				#Need to request with a common UA,
				#else some sites won't give it to you
				r=urllib2.Request(stylesheet)
				mobileUserAgent = MOBILE_UA
				if not config.mobileUserAgent == "":
					mobileUserAgent = config.mobileUserAgent
				r.add_header('User-Agent', mobileUserAgent)
				
				data=urllib2.build_opener().open(r).read()
				
			except Exception as e:
				#If the stylesheet request fails
				if config.isGoogleVersion:
					record.record_error_MediaQueries(e, stylesheet, url, config.errorFileNameGoogleVersion)
				record.record_error_MediaQueries(e, stylesheet, url, config.errorFilename)
				continue    #to the next ss
			
			#Search the stylesheet for media queries
			if re.search('@media', data):
				foundMedia=True
				
				#...and see if they look for width
			if (re.search('<.*media="[^"]*width:.*>', data) or re.search('@media[^}]*width.*{', data)):
				foundMediaWidth=True
		   
	
	return foundMedia, foundMediaWidth

def getMobileVersion(urls, fileLocation):
	remainingURLs = []
	for url in urls:
		'''commented out section is below:'''
		
		log('mediaQueries find %s' % url)
		isMobile = False
		foundMedia = False
		foundMediaWidth = False
		
		content = ""
		js = []
		stylesheets = []
		
		filename2 = ""
		try:
			filename = config.getDomain(url, False, True)#*******************************************************************
			filename = filename + "-fixed"
			filename = fileLocation + filename

			filename2 = filename  + '.css'
			ssfile = open(filename2, 'r')
			ssblock = ssfile.read()
			stylesheets = ssblock.split(',')
			ssfile.close()
			
			filename2 = filename  + '.html'
			htmlfile = open(filename2, 'r')
			content = htmlfile.read()
			htmlfile.close()

			filename2 = filename  + '.js'
			jsfile = open(filename2, 'r')
			jsblock = jsfile.read()
			js = jsblock.split(',')
			jsfile.close()
		except IOError as e:
			log("error opening the fixed files for media queries...")
			print ("failed file name is: " , filename2)
			print "I/O error({0}): {1}".format(e.errno, e.strerror)
		except:
			print "unexpected error"
			log ("\tunexpected error!")

		#Look for media queries in the document
		if (re.search('<link.*media=.*>', content)
			or re.search('@media', content)):
			foundMedia=True
			
			#See if the media queries look for width
			if (re.search('<.*media="[^"]*width:.*>', content)
				or re.search('@media[^}]*width.*{', content)):
				foundMediaWidth=True
	   
		#Look for media queries in document stylesheets
		
		#media queries can be in the js too. and there are inline js and styelsheets that can just be gotten from the content. so need to add those:
		sheets = set()  #so there are no duplicates
		for stylesheet in stylesheets:
			stylesheet = stylesheet.replace('\n', '')
			sheets.add(stylesheet)
		for jscript in js:        
			jscript = jscript.replace('\n', '')
			sheets.add(jscript)
		 
		#look through sheets which has stylesheets AND js files
		foundMedia, foundMediaWidth = findMediaQueriesInStylesheets(sheets, url)
		
		#for inline ss/js: look through each line in content and if it is a link and has .css or .js then it's inline
		#leave alone for now    
		
		if not foundMediaWidth:  #if still haven't found media width then keep looking:
			#search through content for potential media in script tags            
			for line in content: 
				if re.search('@media', line):
					foundMedia=True
					if (re.search('<.*media="[^"]*width:.*>', data) or re.search('@media[^}]*width.*{', data)):
						foundMediaWidth=True
						
		#TODO: look in the sitemaps!!
		''' get from request.py!! look at the headers for /xml to look for media there
		code for sitemaps here
		need to have gotten already from the request.coffee. it is in one of the headers with a content type of text/xml or application/xml
		'''
			
		if config.isGoogleVersion:
			if foundMedia:
				if foundMediaWidth:
					isMobile = True
					record.printGoogleDifferent = True
					record.record_found(url, url, config.foundFilenameGoogleVersion, config.statsFilenameGoogleVersion, 'width') #mobile url is the url since it is just adjusted based on media queries
					record.printGoogleDifferent = False
					#cleanup directories to make room:
					config.cleanupUnusedDirectories (url)
				else:
					record.printGoogleDifferent = True
					record.record_notfound(url, config.notFoundFilenameGoogleVersion, config.statsFilenameGoogleVersion, 'nowidth')
					record.printGoogleDifferent = False
			else:
				record.printGoogleDifferent = True
				record.record_notfound(url, config.notFoundFilenameGoogleVersion, config.statsFilenameGoogleVersion, 'none')
				record.printGoogleDifferent = False
				
		#not google version so just give normal filenames, not google ones   
		if foundMedia:
			if foundMediaWidth:
				isMobile = True
				record.record_found(url, url, config.foundFilename, config.statsFilename, 'width')
				#cleanup directories to make room:
				config.cleanupUnusedDirectories (url)
			else:
				record.record_notfound(url, config.notFoundFilename, config.statsFilename, 'nowidth')
		else: #not foundMedia
			record.record_notfound(url, config.notFoundFilename, config.statsFilename, 'none')
				
		if not isMobile:
			remainingURLs.append(url)
			if config.isGoogleVersion:  #keep the -> to just google messages of why the page was not found
				log ("\t-> no correct media queries found")     
			log ("\tno correct media queries found")   
		
		print "made it to here in mediaQueries with url: " + url + "\n"
	return remainingURLs
	#return urls