'''
Created on May 20, 2013

@author: mwallers
'''

"""

assigns a score for a webpage based on the tags on that page. 
It gathers the popular tags and then tallies up how many times each is present 
on that webpage.
"""

from bs4 import BeautifulSoup
from logger import log
import argparse
import sys

verbose=False

#Define tag weights here
#Total weight may not exceed 1
#Undefined tags will be given equal share of remaining weight
TAG_WEIGHTS={}

def main():
    page1, page2=parse_args()
    print TFDA(page1, page2)
    
def TFDA(page1, page2):
    """Return the TFDA score of two HTML pages"""
    try:
        count1=count_tags(page1)
        count2=count_tags(page2)
    except UnicodeDecodeError as e:
        log(str(e))
        return 0
    
    freq1=tag_frequency(count1)
    freq2=tag_frequency(count2)
    
    tags=set(freq1.keys()+freq2.keys())
    weights=calc_weights(tags)
    
    if verbose:
        for tag in tags:
            tag_freq1=count1.get(tag, 0)
            tag_freq2=count2.get(tag, 0)
        
            sys.stdout.write(tag+'\t')
            sys.stdout.write(str(tag_freq1)+'\t')
            sys.stdout.write(str(tag_freq2)+'\t')
            sys.stdout.write(str(abs(tag_freq1-tag_freq2))+'\n')
    
    distance=0
    for tag in tags:
        difference=freq1.get(tag, 0)-freq2.get(tag, 0)
        distance+=(difference**2)*weights[tag]
        
    return distance
    
def count_tags(page):
    """Return a dict containing tag frequencies for a HTML page"""
    soup=BeautifulSoup(page)
    
    tags={}
    for tag in soup.find_all(True):
        tags[tag.name]=tags.get(tag.name, 0)+1
    
    return tags

def tag_frequency(tags):
    """Normalizes a dict containing tag frequencies to percents"""
    total=sum(tags.values())
    
    freq={}
    for tag, count in tags.iteritems():
        percent=100.0*count/total
        freq[tag]=percent
    
    return freq

def calc_weights(tags):
    """Calculate weights for a squence of tags"""
    #Give an even amount of weight to tags that don't have a defined weight
    weights=dict(TAG_WEIGHTS)
    
    remaining_weight=1.0
    for tag, weight in weights.iteritems():
        if tag not in tags:
            del weights[tag]
            continue
        
        remaining_weight-=weight
    
    if remaining_weight<0:
        raise Exception('The sum of weights may not be over 1')
    
    remaining_tags=tags-set(weights.keys())
    if remaining_tags:
        weight=remaining_weight/len(remaining_tags)
    
        for tag in remaining_tags:
            weights[tag]=weight
    
    return weights

def parse_args():
    parser=argparse.ArgumentParser()
    parser.add_argument('page1', nargs='?', default=None)
    parser.add_argument('page2', nargs='?', default=None)
    
    parser.add_argument('-f', dest='files',
                        type=argparse.FileType('r'),
                        action='append', default=[])
                        
    parser.add_argument('-v', dest='verbose', action='store_true')
    
    args=parser.parse_args()
    
    global verbose
    verbose=args.verbose
    
    page1=None
    page2=None
    
    if args.page1:
        page1=args.page1
    elif args.files:
        page1=args.files.pop().read()
    
    if args.page2:
        page2=args.page2
    elif args.files:
        page2=args.files.pop().read()
    
    if not page1 or not page2:
        parser.print_help()
        sys.exit()
    
    return page1, page2

if __name__=='__main__':
    main()
