import matplotlib.pyplot as plt
import matplotlib.axes as matax
import numpy as np

def graph(values, titles, mainTitle, saveToPath = None, colorsChosen = None):
	if not values or not titles or not mainTitle:
		return  #cannot graph without the essentials
	
	fig = plt.figure()  #makes new instance of a figure for each time this function is called
	
	N = len(values)  #or titles. values and titles should be the same length
		
	ind = np.arange(N)    # the x locations for the groups. just divide the canvas by N
	width = 0.5       # the width of the bars: can also be len(x) sequence
	
	if not colorsChosen:
		colors = ('c', 'b')
	else:
		colors = colorsChosen
		
	pic = plt.bar(ind, values,  width = width, color = colors, edgecolor = 'k')
	
	plt.ylabel('The Number of Urls')
	
	plt.title(mainTitle + '\n', verticalalignment= 'bottom', horizontalalignment= 'center')
	newTitles = []
	for title in titles:
		newTitle = textWrapper(title)
		newTitles.append(newTitle)
		
	plt.subplots_adjust(bottom=0.25) # use a lower number to make more vertical space
	plt.xticks(ind+width/2., newTitles, color = 'k', rotation = 50,stretch = 'ultra-condensed', va = 'top' , ha = 'center')
	plt.yticks(np.arange(0,max(values) + 6,5))
	
	if not saveToPath:
		plt.show()
	else:
		plt.savefig(saveToPath)

def textWrapper(text, cutoff = 15):
	newText = ""
	words = text.split(' ')
	totalLength = 0
	string = ""
	for word in words:
		if len(word) > cutoff: #rare occasion
			newText = word + '\n'
		else:
			totalLength += len(word)
			if totalLength > cutoff:
				#send text so far to newText and then make the word that pushed the limit the begining of a new line
				totalLength = len(word)
				newText += string + '\n'
				string = word
			elif totalLength == cutoff:
				string += word
				newText += string + '\n'
				string = ""
				totalLength = 0
			else:
				string += " " + word
	newText +=string
	return newText
		
if __name__ == '__main__':

	exampleValues = [20, 2, 9, 2, 7, 3]#[11, 3, 8, 1, 4, 2]
	exampleTitles = ["Redirect",	"Stylesheets alone",	"Stylesheets and media queries", 
		"Different content",	"Media queries alone", 
		"Differing urls (trial and error)" ]
	exampleMainTitle = "The methods resulting in locating the mobile version of a desktop page"
	
	graph(exampleValues, exampleTitles, exampleMainTitle, "./graph.png")
	
	exampleValues = [10, 4, 2]
	exampleTitles = ["Vary Http Header missing", "Media width missing", "Missing canonical and/or alternate links"]
	exampleMainTitle = "Tally for 16 urls for which a mobile version was not found"
	colors = ('r','k')
	graph(exampleValues, exampleTitles, exampleMainTitle, "./graph2.png", colors)#C:\\Users\\mwallers\\Documents\\workspace\\mobileFinder\\
	
	
	
	#(transform, valign, halign) = get_yaxis_text1_transform ( 5)
	#set_ylabel(ylabel, **kwargs)